class GetChromeDriverError(Exception):
    pass


class ReleaseUrlError(GetChromeDriverError):
    pass


class UnknownPlatformError(GetChromeDriverError):
    pass


class UnknownReleaseError(GetChromeDriverError):
    pass


class DownloadError(GetChromeDriverError):
    pass


class VersionError(GetChromeDriverError):
    pass


class FeatureNotImplementedError(GetChromeDriverError):
    pass



import os
import requests
import zipfile
import platform as pl
import xml.etree.ElementTree as ElTree
import subprocess
from urllib.request import urlopen
from bs4 import BeautifulSoup

from requests.exceptions import RequestException
from requests.exceptions import HTTPError

from . import constants
from . import retriever
from .platforms import Platforms
from .phase import Phase
from .exceptions import UnknownPlatformError
from .exceptions import ReleaseUrlError
from .exceptions import UnknownReleaseError
from .exceptions import DownloadError
from .exceptions import VersionError
from .exceptions import FeatureNotImplementedError


class GetChromeDriver:

    def __init__(self, platform=None) -> None:
        self.__available_platforms = Platforms()

        if not platform:
            if pl.system() == 'Windows':
                self.__current_set_platform = self.__check_platform(self.__available_platforms.win)
            elif pl.system() == 'Linux':
                self.__current_set_platform = self.__check_platform(self.__available_platforms.linux)
            elif pl.system() == 'Darwin':
                self.__current_set_platform = self.__check_platform(self.__available_platforms.mac)
        else:
            self.__current_set_platform = self.__check_platform(platform)

        self.__phases = Phase()

    def stable_release_version(self) -> str:
        """ Return the latest stable release version """

        return self.__latest_release_version('stable')

    def beta_release_version(self) -> str:
        """ Return the latest beta release version """

        return self.__latest_release_version('beta')

    def __latest_release_version(self, phase) -> str:
        """ Return the stable or beta release version """

        result = requests.get(constants.CHROMEDRIVER_CHROMIUM_URL)
        soup = BeautifulSoup(result.content, 'html.parser')
        ul = soup.select_one(constants.UL_RELEASES_SELECTOR)
        for li in ul:
            li_text = li.text.replace(u'\u00A0', ' ')
            if self.__phases.stable == phase:
                release_str = constants.LATEST_STABLE_RELEASE_STR
            else:
                release_str = constants.LATEST_BETA_RELEASE_STR

            if li_text[:len(release_str)].lower() == release_str.lower():
                try:
                    release = li.a['href'].split('path=')[-1:][0][:-1]
                except TypeError:
                    raise UnknownReleaseError('error: could not find release version')
                else:
                    self.__check_release(release)
                    return release

    def stable_release_url(self) -> str:
        """ Return the latest stable release url """

        return self.release_url(self.__latest_release_version(self.__phases.stable))

    def beta_release_url(self) -> str:
        """ Return the latest beta release url """

        return self.release_url(self.__latest_release_version(self.__phases.beta))

    def release_url(self, release) -> str:
        """ Return the release download url """

        self.__check_release(release)

        url = ''
        if self.__current_set_platform == self.__available_platforms.win:
            url = constants.CHROMEDRIVER_STORAGE_URL + '/' + release + '/' + constants.FILE_NAME_CHROMEDRIVER_WIN_ZIP
        elif self.__current_set_platform == self.__available_platforms.linux:
            url = constants.CHROMEDRIVER_STORAGE_URL + '/' + release + '/' + constants.FILE_NAME_CHROMEDRIVER_LINUX_ZIP
        elif self.__current_set_platform == self.__available_platforms.mac:
            url = constants.CHROMEDRIVER_STORAGE_URL + '/' + release + '/' + constants.FILE_NAME_CHROMEDRIVER_MAC_ZIP

        self.__check_url(url)

        return url

    def download_stable_release(self, output_path=None, extract=False) -> None:
        """ Download the latest stable chromedriver release """

        release = self.__latest_release_version(self.__phases.stable)
        self.download_release(release, output_path, extract)

    def download_beta_release(self, output_path=None, extract=False) -> None:
        """ Download the latest stable chromedriver release """

        release = self.__latest_release_version(self.__phases.beta)
        self.download_release(release, output_path, extract)

    def download_release(self, release, output_path=None, extract=False) -> str:
        """ Download a chromedriver release """

        self.__check_release(release)
        url = self.release_url(release)

        def download(platform_arch):
            if output_path is None:
                output_path_no_file_name = constants.DIR_DOWNLOAD + '/' + release + '/bin'
            else:
                output_path_no_file_name = output_path

            try:
                output_path_with_file_name, file_name = retriever.download(url=url,
                                                                           output_path=output_path_no_file_name)
            except (OSError, HTTPError, RequestException) as err:
                raise DownloadError(err)

            if extract:
                with zipfile.ZipFile(output_path_with_file_name, 'r') as zip_ref:
                    zip_ref.extractall(path=output_path_no_file_name)
                os.remove(output_path_with_file_name)

                if platform_arch == self.__available_platforms.linux_arch:
                    os.chmod(output_path_no_file_name + '/' + constants.FILE_NAME_CHROMEDRIVER, 0o755)

            return output_path_no_file_name

        if self.__current_set_platform == self.__available_platforms.win:
            return download(self.__available_platforms.win_arch)
        elif self.__current_set_platform == self.__available_platforms.linux:
            return download(self.__available_platforms.linux_arch)
        elif self.__current_set_platform == self.__available_platforms.mac:
            return download(self.__available_platforms.mac_arch)

    def __check_url(self, url) -> None:
        """ Check if url is valid """

        if requests.head(url).status_code != 200:
            raise ReleaseUrlError('error: Invalid url')

    def __check_release(self, release) -> None:
        """ Check if release format is valid """

        split_release = release.split('.')

        for number in split_release:
            if not number.isnumeric():
                raise UnknownReleaseError('error: Invalid release format')

    def __check_platform(self, platform) -> str:
        """ Check if platform is valid """

        if platform not in self.__available_platforms.list:
            raise UnknownPlatformError('error: platform not recognized, choose a platform from: '
                                       + str(self.__available_platforms.list))

        return platform

    def auto_download(self, extract=False) -> str:
        """ Download ChromeDriver for the installed Chrome version on machine """

        if pl.system() == 'Darwin':
            raise FeatureNotImplementedError('feature has not been implemented for macOS yet')

        all_chromedriver_versions = self.__get_all_chromedriver_versions()
        installed_chrome_version = self.__get_installed_chrome_version()

        chromedriver_version_to_download = ''
        for chromedriver_version in reversed(all_chromedriver_versions):
            if '.'.join(installed_chrome_version.split('.')[:-1]) == '.'.join(chromedriver_version.split('.')[:-1]):
                chromedriver_version_to_download = chromedriver_version
                break

        if chromedriver_version_to_download == '':
            raise VersionError('error: unable to find a ChromeDriver version that matches the installed Chrome version')

        if pl.system() == 'Windows':
            return self.download_release(chromedriver_version_to_download, extract=extract)

        elif pl.system() == 'Linux':
            return self.download_release(chromedriver_version_to_download, extract=extract)

    def auto_install(self) -> None:
        """ Install ChromeDriver for the installed Chrome version on machine """

        if pl.system() == 'Darwin':
            raise FeatureNotImplementedError('feature has not been implemented for macOS yet')

        output_path = self.auto_download(extract=True)

        if pl.system() == 'Windows':
            path = os.path.join(os.path.abspath(os.getcwd()), output_path)
            os.environ['PATH'] += os.pathsep + os.pathsep.join([path])

        elif pl.system() == 'Linux':
            path = os.path.join(os.path.abspath(os.getcwd()), output_path)
            os.environ['PATH'] += os.pathsep + os.pathsep.join([path])

    def __get_all_chromedriver_versions(self) -> list:
        """ Return a list with all ChromeDriver versions """

        key_texts = []
        versions = []

        url = constants.CHROMEDRIVER_STORAGE_URL

        with urlopen(url) as xml_file:
            tree = ElTree.parse(xml_file)
            root = tree.getroot()

            for root_item in root:
                # Remove namespace
                root_item.tag = root_item.tag.split('}', 1)[1]

            for content in root.findall('Contents'):

                for content_item in content:
                    # Remove namespace
                    content_item.tag = content_item.tag.split('}', 1)[1]

                    key_texts.append(content.find('Key').text)

        for text in key_texts:

            version = ''
            for char in text:
                if char.isnumeric() or char == '.':
                    version += char
                else:
                    break

            if len(version) < 1:
                continue

            versions.append(version)

        return list(dict.fromkeys(versions))

    def __get_installed_chrome_version(self) -> str:
        """ Return the installed Chrome version on the machine """

        if pl.system() == 'Windows':
            process = subprocess.Popen(
                ['reg', 'query', 'HKEY_CURRENT_USER\\SOFTWARE\\Google\\Chrome\\BLBeacon', '/v', 'version'],
                stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, stdin=subprocess.DEVNULL)
            return process.communicate()[0].decode('UTF-8').split()[-1]

        elif pl.system() == 'Linux':
            process = subprocess.Popen(
                ['google-chrome', '--version'],
                stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, stdin=subprocess.DEVNULL)
            return process.communicate()[0].decode('UTF-8').split()[-1]

        elif pl.system() == 'Darwin':
            raise FeatureNotImplementedError('feature has not been implemented for macOS yet')



CHROMEDRIVER_CHROMIUM_URL = 'https://chromedriver.chromium.org'
CHROMEDRIVER_STORAGE_URL = 'https://chromedriver.storage.googleapis.com'
LATEST_RELEASE_URL = CHROMEDRIVER_STORAGE_URL + 'LATEST_RELEASE'

FILE_NAME_CHROMEDRIVER = 'chromedriver'
FILE_NAME_CHROMEDRIVER_EXE = 'chromedriver.exe'
FILE_NAME_CHROMEDRIVER_WIN_ZIP = 'chromedriver_win32.zip'
FILE_NAME_CHROMEDRIVER_LINUX_ZIP = 'chromedriver_linux64.zip'
FILE_NAME_CHROMEDRIVER_MAC_ZIP = 'chromedriver_mac64.zip'

DIR_DOWNLOAD = 'chromedriver'

UL_RELEASES_SELECTOR = '#sites-canvas-main-content > table > tbody > tr > td > div > div:nth-child(5) > ul'

LATEST_STABLE_RELEASE_STR = 'Latest stable release'
LATEST_BETA_RELEASE_STR = 'Latest beta release'



import sys
import argparse
from signal import signal, SIGINT

import colorama

from . import __version__
from . import arguments
from .get_driver import GetChromeDriver
from .platforms import Platforms
from .phase import Phase
from .exceptions import GetChromeDriverError


def main():
    # noinspection PyUnusedLocal
    def signal_handler(signal_received, frame):
        """ Handles clean Ctrl+C exit """

        sys.stdout.write('\n')
        sys.exit(0)

    signal(SIGINT, signal_handler)
    App()


class App:

    def __init__(self):
        self.__c_fore = colorama.Fore
        self.__c_style = colorama.Style
        colorama.init()

        self.__platforms = Platforms()
        self.__phases = Phase()

        self.__msg_download_finished = 'download finished'
        self.__msg_required_add_release = (self.__c_fore.RED + 'required: add a release version'
                                           + self.__c_style.RESET_ALL)
        self.__msg_optional_add_extract = 'optional: add --extract to extract the zip file'
        self.__msg_error_unrecognized_argument = (
                self.__c_fore.RED + 'error: unrecognized argument(s) detected' + self.__c_style.RESET_ALL
                + '\n' + 'tip: use --help to see all available arguments')
        self.__msg_download_error = (self.__c_fore.RED + 'error: an error occurred at downloading'
                                     + self.__c_style.RESET_ALL)
        self.__msg_release_url_error = (self.__c_fore.RED + 'error: could not find release url'
                                        + self.__c_style.RESET_ALL)
        self.__msg_no_stable_found_error = (self.__c_fore.RED + 'not found'
                                            + self.__c_style.RESET_ALL)
        self.__msg_no_beta_available_error = (self.__c_fore.RED + 'not beta version available'
                                              + self.__c_style.RESET_ALL)
        self.__msg_no_beta_release_version_error = (self.__c_fore.RED
                                                    + 'error: could not find a beta release version'
                                                    + self.__c_style.RESET_ALL)
        self.__msg_no_stable_release_version_error = (self.__c_fore.RED
                                                      + 'error: could not find a stable release version'
                                                      + self.__c_style.RESET_ALL)

        self.__parser = argparse.ArgumentParser(add_help=False)
        for i, arg in enumerate(arguments.args_options):
            self.__parser.add_argument(arguments.args_options[i][0], nargs='*')
        self.__args, self.__unknown = self.__parser.parse_known_args()

        if self.__unknown:
            print(self.__msg_error_unrecognized_argument)
            sys.exit(0)

        ###################
        # DEFAULT NO ARGS #
        ###################
        if len(sys.argv) == 1:
            arguments.print_help()
            sys.exit(0)

        ########
        # HELP #
        ########
        self.__arg_help = self.__args.help
        if self.__arg_passed(self.__arg_help):
            arguments.print_help()
            sys.exit(0)

        ################
        # BETA VERSION #
        ################
        self.__arg_beta_version = self.__args.beta_version
        if self.__arg_passed(self.__arg_beta_version):
            self.print_phase_version(self.__phases.beta)
            sys.exit(0)

        ##################
        # STABLE VERSION #
        ##################
        self.__arg_stable_version = self.__args.stable_version
        if self.__arg_passed(self.__arg_stable_version):
            self.print_phase_version(self.__phases.stable)
            sys.exit(0)

        ###############
        # LATEST URLS #
        ###############
        self.__arg_latest_urls = self.__args.latest_urls
        if self.__arg_passed(self.__arg_latest_urls):
            self.print_urls()
            sys.exit(0)

        ###############
        # RELEASE URL #
        ###############
        self.__arg_release_url = self.__args.release_url
        if self.__arg_passed(self.__arg_release_url):
            if len(self.__arg_release_url) < 1:
                print(self.__msg_required_add_release)
            else:
                self.print_release_url(self.__arg_release_url[0])
            sys.exit(0)

        ############
        # BETA URL #
        ############
        self.__arg_beta_url = self.__args.beta_url
        if self.__arg_passed(self.__arg_beta_url):
            self.print_phase_url(self.__phases.beta)
            sys.exit(0)

        ##############
        # STABLE URL #
        ##############
        self.__arg_stable_url = self.__args.stable_url
        if self.__arg_passed(self.__arg_stable_url):
            self.print_phase_url(self.__phases.stable)
            sys.exit(0)

        #################
        # AUTO DOWNLOAD #
        #################
        self.__arg_auto_download = self.__args.auto_download
        if self.__arg_passed(self.__arg_auto_download):
            extract = False
            self.__arg_extract = self.__args.extract
            if self.__arg_passed(self.__arg_extract):
                extract = True
            self.auto_download(extract)
            sys.exit(0)

        #################
        # DOWNLOAD BETA #
        #################
        self.__arg_download_beta = self.__args.download_beta
        if self.__arg_passed(self.__arg_download_beta):
            extract = False
            self.__arg_extract = self.__args.extract
            if self.__arg_passed(self.__arg_extract):
                extract = True
            self.download_phase_release(self.__phases.beta, extract)

        ###################
        # DOWNLOAD STABLE #
        ###################
        self.__arg_download_stable = self.__args.download_stable
        if self.__arg_passed(self.__arg_download_stable):
            extract = False
            self.__arg_extract = self.__args.extract
            if self.__arg_passed(self.__arg_extract):
                extract = True
            self.download_phase_release(self.__phases.stable, extract)

        ####################
        # DOWNLOAD RELEASE #
        ####################
        self.__arg_download_release = self.__args.download_release
        if self.__arg_passed(self.__arg_download_release):
            extract = False
            self.__arg_extract = self.__args.extract
            if self.__arg_passed(self.__arg_extract):
                extract = True
            if len(self.__arg_download_release) < 1:
                print(self.__msg_required_add_release)
                print(self.__msg_optional_add_extract)
                sys.exit(0)
            else:
                release = self.__arg_download_release[0]
                self.download_release(release, extract)
            sys.exit(0)

        ###########
        # VERSION #
        ###########
        self.__arg_version = self.__args.version
        if self.__arg_passed(self.__arg_version):
            print('v' + __version__)
            sys.exit(0)

    def __arg_passed(self, arg):
        """ Check if arguments were passed """

        if isinstance(arg, list):
            return True
        return False

    def print_urls(self):
        """ Print the stable nd beta url release for all platforms """

        get_driver_win = GetChromeDriver(self.__platforms.win)
        get_driver_linux = GetChromeDriver(self.__platforms.linux)
        get_driver_mac = GetChromeDriver(self.__platforms.mac)
        drivers = {'Windows': get_driver_win, 'Linux': get_driver_linux, 'macOS': get_driver_mac}

        for index, (key, value) in enumerate(drivers.items()):
            print('Latest beta and stable release for ' + key + ': ')
            try:
                print('stable : ' + value.stable_release_url())
            except GetChromeDriverError:
                print(self.__msg_no_stable_found_error)
            try:
                print('beta   : ' + value.beta_release_url())
            except GetChromeDriverError:
                print(self.__msg_no_beta_available_error)

            if index < len(drivers) - 1:
                print('')

    def print_phase_version(self, phase):
        """ Print stable version or beta version """

        get_driver = GetChromeDriver()

        if phase == self.__phases.beta:
            try:
                print(get_driver.beta_release_version())
            except GetChromeDriverError:
                print(self.__msg_no_beta_release_version_error)
        else:
            try:
                print(get_driver.stable_release_version())
            except GetChromeDriverError:
                print(self.__msg_no_stable_release_version_error)

    def print_phase_url(self, phase):
        """ Print stable url or beta url """

        get_driver = GetChromeDriver()
        if phase == self.__phases.beta:
            try:
                print(get_driver.beta_release_url())
            except GetChromeDriverError:
                print(self.__msg_release_url_error)
        else:
            try:
                print(get_driver.stable_release_url())
            except GetChromeDriverError:
                print(self.__msg_release_url_error)

    def print_release_url(self, release):
        """ Print the url for a given version """

        get_driver = GetChromeDriver()

        try:
            print(get_driver.release_url(release))
        except GetChromeDriverError:
            print(self.__msg_release_url_error)

    def auto_download(self, extract):
        """ Auto download ChromeDriver """

        get_driver = GetChromeDriver()

        try:
            get_driver.auto_download(extract=extract)
            print(self.__msg_download_finished)
        except GetChromeDriverError:
            print(self.__msg_download_error)

    def download_phase_release(self, phase, extract):
        """ Download the release for the stable version or beta version """

        get_driver = GetChromeDriver()

        if phase == self.__phases.beta:
            try:
                get_driver.download_beta_release(extract=extract)
                print(self.__msg_download_finished)
            except GetChromeDriverError:
                print(self.__msg_download_error)
                print(self.__msg_no_beta_release_version_error)
        else:
            try:
                get_driver.download_stable_release(extract=extract)
                print(self.__msg_download_finished)
            except GetChromeDriverError:
                print(self.__msg_download_error)

    def download_release(self, release, extract):
        """ Download the release of a given version """

        get_driver = GetChromeDriver()

        try:
            get_driver.download_release(release, extract=extract)
            print(self.__msg_download_finished)
        except GetChromeDriverError:
            print(self.__msg_download_error)



import os
import requests
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from urllib.parse import urlparse
from requests.exceptions import RequestException
from requests.exceptions import HTTPError


def download(url, output_path=None, file_name=None):
    """
    Download a file from url
    If output_path is None, the file will be downloaded directly in the current directory
    """

    session = __retry_session(retries=3,
                              backoff_factor=0.1,
                              status_forcelist=[429, 500, 502, 503, 504],
                              method_whitelist=['GET'])
    try:
        res = session.get(url=url)
    except RequestException as err:
        raise RequestException(err)
    else:
        if res.status_code != 200:
            raise HTTPError('Invalid URL')

        if file_name is None:
            file_name = get_file_name_from_url(url)

        if output_path is None:
            output_path = file_name
        else:
            __create_dir(output_path)
            output_path = output_path + '/' + file_name

        with open(output_path, 'wb') as file:
            for chunk in res.iter_content(chunk_size=1048576):
                if chunk:
                    file.write(chunk)
        return output_path, file_name
    finally:
        session.close()


def __retry_session(retries, backoff_factor, status_forcelist, method_whitelist):
    """ Retry session """

    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
        method_whitelist=method_whitelist)

    adapter = HTTPAdapter(max_retries=retry)
    session = requests.Session()
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session


def get_file_name_from_url(url):
    """ Get file name from url """

    path = urlparse(url).path
    return path.split('/')[-1]


def __create_dir(directory):
    """ Create a directory """

    try:
        os.makedirs(directory, exist_ok=True)
    except OSError as err:
        raise OSError(err)




from .platforms import Platforms

platforms = Platforms()

descriptor = '  {:<30} {}'
message_help_required_release = descriptor.format('', 'required: add a release version')
message_help_optional_extract = descriptor.format('', 'optional: use --extract to extract the zip file')

args_options = [
    ['--beta-version', 'print the beta release version'],
    ['--stable-version', 'print the stable release version'],
    ['--latest-urls', 'print the beta and stable release urls for all platforms'],
    ['--release-url', 'print the url of a release for a platform' + '\n'
     + message_help_required_release],
    ['--beta-url', 'print the beta release url for a platform'],
    ['--stable-url', 'print the stable release url for a platform'],
    ['--auto-download', 'auto download ChromeDriver for the installed Chrome Version' + '\n'
     + message_help_optional_extract],
    ['--download-beta', 'download the beta release for a platform' + '\n'
     + message_help_optional_extract],
    ['--download-stable', 'download the stable release for a platform' + '\n'
     + message_help_optional_extract],
    ['--download-release', 'download a release' + '\n'
     + message_help_required_release + '\n'
     + message_help_optional_extract],
    ['--extract', 'option to extract the zip file'],
    ['--version', 'program version'],
    ['--help', 'show help']
]


def print_help():
    print('usage: ' + 'get-chrome-driver' + ' [options]')
    print('')
    print('options: ')
    for i, argument in enumerate(args_options):
        print(descriptor.format(argument[0], argument[1]))
    print('')
    print('The downloaded driver can be found at: ')
    print('<current directory>/<chromedriver>/<version>/<bin>/<chromedriver>')



class Platforms:

    def __init__(self):
        self.win = self.__win()
        self.linux = self.__linux()
        self.mac = self.__mac()
        self.list = self.__list()

        self.win_arch = self.__win_arch()
        self.linux_arch = self.__linux_arch()
        self.mac_arch = self.__mac_arch()

    def __win(self):
        return 'win'

    def __linux(self):
        return 'linux'

    def __mac(self):
        return 'mac'

    def __win_arch(self):
        return 'win32'

    def __linux_arch(self):
        return 'linux64'

    def __mac_arch(self):
        return 'mac64'

    def __list(self):
        return [self.__win(), self.__linux(), self.__mac()]



from setuptools import setup
from setuptools import find_packages

name = 'get-chrome-driver'
version = '1.1.25'

with open('README.md', 'r') as fh:
    long_description = fh.read()

requires = [
    'bs4>=0.0.1',
    'requests>=2.24.0',
    'colorama>=0.4.3',
    'urllib3>=1.25.10'
]

setup(
    name=name,
    version=version,
    author='Zairon Jacobs',
    author_email='zaironjacobs@gmail.com',
    description='A tool to download and install ChromeDriver.',
    long_description=long_description,
    url='https://github.com/zaironjacobs/get-chrome-driver',
    download_url='https://github.com/zaironjacobs/get-chrome-driver/archive/v' + version + '.tar.gz',
    keywords=['chrome', 'chromedriver', 'download', 'install', 'web', 'driver', 'tool', 'get'],
    packages=find_packages(),
    entry_points={
        'console_scripts': [name + '=get_chrome_driver.app:main'],
    },
    install_requires=requires,
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Natural Language :: English'
    ],
    python_requires='>=3',
)



import pytest
import os
import shutil
import subprocess
import platform as pl
from os import path

from bs4 import BeautifulSoup
import requests
from decouple import config

from .. import constants
from .. import __version__
from ..platforms import Platforms

name = 'get-chrome-driver'

platforms = Platforms()

stable_release = config('STABLE_RELEASE')
random_release = config('RANDOM_RELEASE')

file_name_zipped = ''
stable_release_url = ''
random_release_url = ''
if pl.system() == 'Windows':
    file_name_zipped = 'chromedriver_win32.zip'
    file_name = 'chromedriver.exe'
    stable_release_url = 'https://chromedriver.storage.googleapis.com/' + stable_release + '/chromedriver_win32.zip'
    random_release_url = 'https://chromedriver.storage.googleapis.com/' + random_release + '/chromedriver_win32.zip'
elif pl.system() == 'Linux':
    file_name_zipped = 'chromedriver_linux64.zip'
    file_name = 'chromedriver'
    stable_release_url = 'https://chromedriver.storage.googleapis.com/' + stable_release + '/chromedriver_linux64.zip'
    random_release_url = 'https://chromedriver.storage.googleapis.com/' + random_release + '/chromedriver_linux64.zip'
elif pl.system() == 'Darwin':
    file_name_zipped = 'chromedriver_mac64.zip'
    file_name = 'chromedriver'
    stable_release_url = 'https://chromedriver.storage.googleapis.com/' + stable_release + '/chromedriver_mac64.zip'
    random_release_url = 'https://chromedriver.storage.googleapis.com/' + random_release + '/chromedriver_mac64.zip'

# Change to the current test directory
os.chdir(os.path.dirname(__file__))


class TestApp:

    ###################################
    # LI TEXT "LATEST STABLE RELEASE" #
    ###################################
    def test_text_match_latest_stable(self):
        match_found = False

        result = requests.get(constants.CHROMEDRIVER_CHROMIUM_URL)
        soup = BeautifulSoup(result.content, 'html.parser')
        ul = soup.select_one(constants.UL_RELEASES_SELECTOR)
        for li in ul:
            text = li.text.replace(u'\u00A0', ' ')
            if text[:len(constants.LATEST_STABLE_RELEASE_STR)].lower() == constants.LATEST_STABLE_RELEASE_STR.lower():
                match_found = True
                break

        assert match_found is True

    #################################
    # LI TEXT "LATEST BETA RELEASE" #
    #################################
    def test_text_match_latest_beta(self):
        match_found = False

        result = requests.get(constants.CHROMEDRIVER_CHROMIUM_URL)
        soup = BeautifulSoup(result.content, 'html.parser')
        ul = soup.select_one(constants.UL_RELEASES_SELECTOR)
        for li in ul:
            text = li.text.replace(u'\u00A0', ' ')
            if text[:len(constants.LATEST_BETA_RELEASE_STR)].lower() == constants.LATEST_BETA_RELEASE_STR.lower():
                match_found = True
                break

        assert match_found is True

    ##################
    # STABLE VERSION #
    ##################
    def test_stable_release_version(self):
        out = subprocess.run(args=[name, '--stable-version'],
                             universal_newlines=True,
                             stdout=subprocess.PIPE)
        actual = out.stdout.split()[0]
        assert stable_release == str(actual)

    ######################
    # RANDOM RELEASE URL #
    ######################
    def test_random_win_release_url(self):
        url = random_release_url
        out = subprocess.run(args=[name, '--release-url', random_release],
                             universal_newlines=True,
                             stdout=subprocess.PIPE)
        actual = out.stdout.split()[0]
        assert url, str(actual)

    ######################
    # STABLE RELEASE URL #
    ######################
    def test_stable_release_url(self):
        url = stable_release_url
        out = subprocess.run(args=[name, '--stable-url'],
                             universal_newlines=True,
                             stdout=subprocess.PIPE)
        actual = out.stdout.split()[0]
        assert url == str(actual)

    ##############################
    # AUTO DOWNLOAD - NO EXTRACT #
    ##############################
    def test_auto_download_no_extract(self):
        # Keep Chrome on test machine updated to match the stable ChromeDriver during testing

        release = stable_release
        subprocess.run(args=[name, '--auto-download'], stdout=subprocess.PIPE)
        file_path = (constants.DIR_DOWNLOAD + '/' + release + '/'
                     + 'bin' + '/' + file_name_zipped)
        result = path.exists(file_path)
        assert result

    ###########################
    # AUTO DOWNLOAD - EXTRACT #
    ###########################
    def test_auto_download_extract(self):
        # Keep Chrome on test machine updated to match the stable ChromeDriver during testing

        release = stable_release
        subprocess.run(args=[name, '--auto-download', '--extract'], stdout=subprocess.PIPE)
        file_path = (constants.DIR_DOWNLOAD + '/' + release + '/'
                     + 'bin' + '/' + file_name)
        result = path.exists(file_path)
        assert result

    ########################################
    # DOWNLOAD STABLE RELEASE - NO EXTRACT #
    ########################################
    def test_download_stable_release_no_extract(self):
        release = stable_release
        subprocess.run(args=[name, '--download-stable'], stdout=subprocess.PIPE)
        file_path = (constants.DIR_DOWNLOAD + '/' + release + '/'
                     + 'bin' + '/' + file_name_zipped)
        result = path.exists(file_path)
        assert result

    #######################################
    # DOWNLOAD STABLE RELEASE - EXTRACTED #
    #######################################
    def test_download_stable_win_release_extract(self):
        release = stable_release
        subprocess.run(args=[name, '--download-stable', '--extract'], stdout=subprocess.PIPE)
        file_path_extracted = (constants.DIR_DOWNLOAD + '/' + release + '/'
                               + 'bin' + '/' + file_name)
        result = path.exists(file_path_extracted)
        assert result

    ########################################
    # DOWNLOAD RANDOM RELEASE - NO EXTRACT #
    ########################################
    def test_download_random_win_release_no_extract(self):
        release = random_release
        subprocess.run(args=[name, '--download-release', release], stdout=subprocess.PIPE)
        file_path = (constants.DIR_DOWNLOAD + '/' + release + '/'
                     + 'bin' + '/' + file_name_zipped)
        result = path.exists(file_path)
        assert result

    #######################################
    # DOWNLOAD RANDOM RELEASE - EXTRACTED #
    #######################################
    def test_download_random_win_release_extract(self):
        release = random_release
        subprocess.run(args=[name, '--download-release', release, '--extract'],
                       stdout=subprocess.PIPE)
        file_path_extracted = (constants.DIR_DOWNLOAD + '/' + release + '/'
                               + 'bin' + '/' + file_name)
        result = path.exists(file_path_extracted)
        assert result

    ###########
    # VERSION #
    ###########
    def test_version(self):
        out = subprocess.run(args=[name, '--version'],
                             universal_newlines=True,
                             stdout=subprocess.PIPE)
        actual = out.stdout.split()[0]
        assert 'v' + __version__ == str(actual)

    ###########
    # CLEANUP #
    ###########
    @pytest.fixture(scope='session', autouse=True)
    def cleanup(self):
        yield
        try:
            shutil.rmtree('chromedriver')
        except FileNotFoundError:
            pass
